package com.example.wc24h;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.support.v7.app.AppCompatActivity;
import android.text.Html;
import android.util.Log;
import android.util.Xml;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import android.widget.Toast;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.URL;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

public class MainActivity extends AppCompatActivity
{
	public volatile byte[]				global_messageBytes;
	public volatile int					global_msg_length;
	public WC24hSettings                wc24hsettings = new WC24hSettings();
	private SeekBar						redBrightness;
	private SeekBar						greenBrightness;
	private SeekBar						blueBrightness;
	private SeekBar						whiteBrightness;
	private SeekBar						brightness;
	private CheckBox					automaticBrightness;

    private static final int			MenuEditServer	= Menu.FIRST + 1;
    private static final int			MenuSync    	= Menu.FIRST + 2;
	private static final int			MenuAbout		= Menu.FIRST + 3;
	private static final int			MenuExit		= Menu.FIRST + 4;
    private static final int			MenuTetris		= Menu.FIRST + 5;
	private MenuItem					MenuItemEditServer;
	private boolean						menusCreated = false;
	Resources							r;
    private EditText					tvae;
    private ImageButton					refreshSettingsButton;
    TextView                            displayModeText;
    TextView                            animationModeText;
    TextView                            colorAnimationModeText;
    LinearLayout                        rgbw_controls;
    TextView                            profileText;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_NOSENSOR);

        // create settings directory if needed
        getSettingsDirFile(this);

        setContentView(R.layout.activity_main);

        rgbw_controls = (LinearLayout) findViewById(R.id.rgbw_controls);

        redBrightness = (SeekBar) findViewById(R.id.RedColor);
        redBrightness.setMax(63);
        redBrightness.setOnSeekBarChangeListener(new seekListener());

        greenBrightness = (SeekBar) findViewById(R.id.GreenColor);
        greenBrightness.setMax(63);
        greenBrightness.setOnSeekBarChangeListener(new seekListener());

		blueBrightness = (SeekBar) findViewById(R.id.BlueColor);
		blueBrightness.setMax(63);
		blueBrightness.setOnSeekBarChangeListener(new seekListener());

		whiteBrightness = (SeekBar) findViewById(R.id.WhiteColor);

		brightness = (SeekBar) findViewById(R.id.Brightness);
        brightness.setMax(15);
        brightness.setOnSeekBarChangeListener(new seekListener());
        
        automaticBrightness = (CheckBox) findViewById(R.id.AutomaticBrightness);

        /************
         * display mode
         ************/
        LinearLayout l = (LinearLayout) findViewById(R.id.dsp_mode_selector);
        ((TextView)l.findViewById(R.id.description)).setText(R.string.Display_mode_);
        displayModeText = (TextView)l.findViewById(R.id.text);
        ImageButton ib = (ImageButton)l.findViewById(R.id.select_btn);
        ib.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                final SelectionDialog dlg = new SelectionDialog(MainActivity.this);
                ArrayList<String> modes = new ArrayList<String>();
                for (String str: wc24hsettings.display_modes) {
                    modes.add(str);
                }
                dlg.setSelections(modes);
                dlg.setSelected(wc24hsettings.getNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR));
                dlg.setDialogTitle(R.string.Display_mode);
                dlg.setSelectedListener(new SelectionDialog.SelectionDialogListener() {
                    @Override
                    public void buttonPressed(int selected) {
                        if (selected >= 0) {
                            wc24hsettings.setNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR, selected);
                            if (wc24hsettings.display_modes.size() > selected) {
                                displayModeText.setText(wc24hsettings.display_modes.elementAt(selected));
                            } else {
                                displayModeText.setText("-");
                            }

                            byte[] messageBytes = new byte[2];
                            messageBytes[0] = 'D';
                            messageBytes[1] = (byte) selected;
                            SendMessage(messageBytes, 2);
                            Toast.makeText(getBaseContext(), R.string.display_mode_sent, Toast.LENGTH_SHORT).show();
                            dlg.dismiss();
                        }
                    }
                });
                dlg.show();
            }
        });

        /************
         * animation mode
         ************/
        l = (LinearLayout) findViewById(R.id.animation_mode_selector);
        ((TextView)l.findViewById(R.id.description)).setText(R.string.Animation);
        animationModeText = (TextView)l.findViewById(R.id.text);
        ib = (ImageButton)l.findViewById(R.id.select_btn);
        ib.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                final SelectionDialog dlg = new SelectionDialog(MainActivity.this);
                ArrayList<String> modes = new ArrayList<String>();
                for (WC24hSettings.Animation animation: wc24hsettings.display_animations) {
                    modes.add(animation.name);
                }
                dlg.setSelections(modes);
                dlg.setSelected(wc24hsettings.getNumVar(WC24hSettings.ANIMATION_MODE_NUM_VAR));
                dlg.setDialogTitle(R.string.Animation);
                dlg.setSelectedListener(new SelectionDialog.SelectionDialogListener() {
                    @Override
                    public void buttonPressed(int selected) {
                        if (selected >= 0) {
                            wc24hsettings.setNumVar(WC24hSettings.ANIMATION_MODE_NUM_VAR, selected);
                            if (wc24hsettings.display_animations.size() > selected) {
                                animationModeText.setText(wc24hsettings.display_animations.elementAt(selected).name);
                            } else {
                                animationModeText.setText("-");
                            }

                            byte[] messageBytes = new byte[2];
                            messageBytes[0] = 'A';
                            messageBytes[1] = (byte) selected;
                            SendMessage(messageBytes, 2);
                            Toast.makeText(getBaseContext(), R.string.animation_mode_sent, Toast.LENGTH_SHORT).show();
                            dlg.dismiss();
                        }
                    }
                });
                dlg.show();
            }
        });

        /************
         * color animation mode
         ************/
        l = (LinearLayout) findViewById(R.id.color_animation_mode_selector);
        ((TextView)l.findViewById(R.id.description)).setText(R.string.Color_animation);
        colorAnimationModeText = (TextView)l.findViewById(R.id.text);
        ib = (ImageButton)l.findViewById(R.id.select_btn);
        ib.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                final SelectionDialog dlg = new SelectionDialog(MainActivity.this);
                ArrayList<String> modes = new ArrayList<String>();
                for (WC24hSettings.Animation animation: wc24hsettings.color_animations) {
                    modes.add(animation.name);
                }
                dlg.setSelections(modes);
                dlg.setSelected(wc24hsettings.getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR));
                dlg.setDialogTitle(R.string.Color_animation);
                dlg.setSelectedListener(new SelectionDialog.SelectionDialogListener() {
                    @Override
                    public void buttonPressed(int selected) {
                        if (selected >= 0) {
                            wc24hsettings.setNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR, selected);
                            if (wc24hsettings.color_animations.size() > selected) {
                                colorAnimationModeText.setText(wc24hsettings.color_animations.elementAt(selected).name);
                            } else {
                                colorAnimationModeText.setText("-");
                            }

                            if (selected == 0) {
                                rgbw_controls.setVisibility(View.VISIBLE);
                            } else {
                                rgbw_controls.setVisibility(View.GONE);
                            }

                            byte[] messageBytes = new byte[2];
                            messageBytes[0] = 'F';
                            messageBytes[1] = (byte) selected;
                            SendMessage(messageBytes, 2);
                            Toast.makeText(getBaseContext(), R.string.color_animation_mode_sent, Toast.LENGTH_SHORT).show();
                            dlg.dismiss();
                        }
                    }
                });
                dlg.show();
            }
        });

        refreshSettingsButton = (ImageButton) findViewById(R.id.refreshSettingsButton);
        refreshSettingsButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                new SettingsGetter(MainActivity.this, wc24hsettings).execute();
            }
        });

        profileText = (TextView)findViewById(R.id.profile_text);

        profileChanged(false);
    }

    @Override
    public void onStart() {
        super.onStart();
        if (!isNetworkAvailable(this)) {
            InfoDialog info = new InfoDialog(this, R.string.no_network_info);
            info.show();

        } else {
            new SettingsGetter(MainActivity.this, wc24hsettings).execute();
        }
    }

    public boolean isNetworkAvailable(Context context) {
        ConnectivityManager connectivity = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);

        if (connectivity == null) {
            return false;
        } else {
            NetworkInfo[] info = connectivity.getAllNetworkInfo();
            if (info != null) {
                for (NetworkInfo anInfo : info) {
                    if (anInfo.getState() == NetworkInfo.State.CONNECTED) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    void updateView(int profile)
    {
        WC24hSettings.DisplayColor color = wc24hsettings.getColor(0);

        redBrightness.setProgress(color.red);
        greenBrightness.setProgress(color.green);
        blueBrightness.setProgress(color.blue);
        if (wc24hsettings.isRGBW())
        {
            whiteBrightness.setProgress(color.white);
        }
        brightness.setProgress(wc24hsettings.getNumVar(WC24hSettings.DISPLAY_BRIGHTNESS_NUM_VAR));
        brightness.setProgress(wc24hsettings.getNumVar(WC24hSettings.DISPLAY_BRIGHTNESS_NUM_VAR));
        automaticBrightness.setChecked(wc24hsettings.getNumVar(WC24hSettings.DISPLAY_AUTOMATIC_BRIGHTNESS_ACTIVE_NUM_VAR) != 0);
        if (wc24hsettings.display_modes.size() > wc24hsettings.getNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR)) {
            displayModeText.setText(wc24hsettings.display_modes.elementAt(wc24hsettings.getNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR)));
        } else {
            displayModeText.setText("-");
        }
        if (wc24hsettings.display_animations.size() > wc24hsettings.getNumVar(WC24hSettings.ANIMATION_MODE_NUM_VAR)) {
            animationModeText.setText(wc24hsettings.display_animations.elementAt(wc24hsettings.getNumVar(WC24hSettings.ANIMATION_MODE_NUM_VAR)).name);
        } else {
            animationModeText.setText("-");
        }
        if (wc24hsettings.color_animations.size() > wc24hsettings.getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR)) {
            colorAnimationModeText.setText(wc24hsettings.color_animations.elementAt(wc24hsettings.getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR)).name);
            if (wc24hsettings.getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR) == 0) {
                rgbw_controls.setVisibility(View.VISIBLE);
            } else {
                rgbw_controls.setVisibility(View.GONE);
            }
        } else {
            colorAnimationModeText.setText("-");
        }
        File f = new File(getSettingsDirFile(this), MainActivity.getProfileFileName(profile));
        String date = "--:--:--";
        String ip = wc24hsettings.ipaddress;
        String descr = wc24hsettings.description;
        if (f.exists()) {
            Date d = new Date(f.lastModified());
            DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
            date = df.format(d);
        }

        profileText.setText(Html.fromHtml(getString(R.string.profile_info, (profile + 1), date, ip, descr)));
    }

    void profileChanged(boolean isRefreshed)
    {
        SharedPreferences prefs = getSharedPreferences("com.example.wc24h", Context.MODE_PRIVATE);
        int selectedProfile = prefs.getInt("com.example.wc24h.selected_profile", 0);
        Log.w("WC24h", "Profile: " + selectedProfile);
        if (!isRefreshed) {
            wc24hsettings.loadData(getSettingsDirFile(this), getProfileFileName(selectedProfile));
        }

        if (wc24hsettings.isRGBW())
        {
            TextView tv = (TextView) findViewById(R.id.tvWhiteColor);
            tv.setVisibility(View.VISIBLE);

            whiteBrightness.setVisibility(View.VISIBLE);
            whiteBrightness.setMax(63);
            whiteBrightness.setOnSeekBarChangeListener(new seekListener());
        }
        else
        {
            TextView tv = (TextView) findViewById(R.id.tvWhiteColor);
            tv.setVisibility(View.GONE);

            whiteBrightness.setVisibility(View.GONE);
        }

        updateView(selectedProfile);
    }

    void sendDisplayColors()
    {
        byte[] messageBytes = new byte[5];
        WC24hSettings.DisplayColor color = wc24hsettings.getColor(0);
        messageBytes[0] = 'C';
        messageBytes[1] = (byte) color.red;
        messageBytes[2] = (byte) color.green;
        messageBytes[3] = (byte) color.blue;

        if (wc24hsettings.isRGBW())
        {
            messageBytes[4] = (byte) color.white;
            SendMessage (messageBytes, 5);
        }
        else
        {
            SendMessage (messageBytes, 4);
        }
    }

    void sendBrightness()
    {
        byte[] messageBytes = new byte[2];
        int b = wc24hsettings.getNumVar(WC24hSettings.DISPLAY_BRIGHTNESS_NUM_VAR);
        messageBytes[0] = 'B';
        messageBytes[1] = (byte) b;
        SendMessage (messageBytes, 2);
        wc24hsettings.setNumVar(WC24hSettings.DISPLAY_AUTOMATIC_BRIGHTNESS_ACTIVE_NUM_VAR, 0/*false*/);
    }

    private class seekListener implements OnSeekBarChangeListener
	{
	    @Override
	    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser)
	    {
	    }

	    @Override
	    public void onStartTrackingTouch(SeekBar seekBar)
	    {
	    }

	    @Override
	    public void onStopTrackingTouch(SeekBar seekBar)
	    {
	        //byte[] messageBytes = new byte[5];

	        switch (seekBar.getId())
			{
				case R.id.Brightness:
                    int b = brightness.getProgress ();
                    wc24hsettings.setNumVar(WC24hSettings.DISPLAY_BRIGHTNESS_NUM_VAR, b);
                    sendBrightness();
					automaticBrightness.setChecked (false);
					Toast.makeText(getBaseContext(), R.string.brightness_sent, Toast.LENGTH_SHORT).show();
					break;
				default:
                    WC24hSettings.DisplayColor color = wc24hsettings.getColor(0);
                    color.red	= (byte) redBrightness.getProgress ();
                    color.green	= (byte) greenBrightness.getProgress ();
                    color.blue	= (byte) blueBrightness.getProgress ();
                    color.white	= (byte) whiteBrightness.getProgress ();
                    wc24hsettings.setColor(0, color);
				    sendDisplayColors();
                    Toast.makeText(getBaseContext(), R.string.colors_are_sent, Toast.LENGTH_SHORT).show();
			}

	    }
	}

	public void SendMessage (byte[] messageBytes, int msg_length)
	{
		global_messageBytes = messageBytes;
		global_msg_length = msg_length;
		new CommandSender().run();
	}

	public void myClickHandler(View view)
	{

		switch (view.getId())
		{
			case R.id.PowerOnButton:
			{
				byte[] messageBytes = new byte[2];
				messageBytes[0] = 'P';
				messageBytes[1] = 1;
				SendMessage (messageBytes, 2);
				Toast.makeText(getBaseContext(), R.string.power_on_sent, Toast.LENGTH_SHORT).show();
				break;
			}
			case R.id.PowerOffButton:
			{
				byte[] messageBytes = new byte[2];
				messageBytes[0] = 'P';
				messageBytes[1] = 0;
				SendMessage (messageBytes, 2);
				Toast.makeText(getBaseContext(), R.string.power_off_sent, Toast.LENGTH_SHORT).show();
				break;
			}
			case R.id.TemperatureButton:
			{
				byte[] messageBytes = new byte[1];
				messageBytes[0] = 'W';
				SendMessage (messageBytes, 1);
				Toast.makeText(getBaseContext(), R.string.show_temp_sent, Toast.LENGTH_SHORT).show();
				break;
			}
            case R.id.Profilesbutton:
            {
                ProfilesDialog profilesDialog = new ProfilesDialog(this);
                profilesDialog.show();
                break;
            }
        }
	}

    private class SettingsGetter extends AsyncTask<String, Integer, Long>
    {
        WC24hSettings settings;
        MainActivity mainActivity;
        SettingsGetter(MainActivity mainActivity, WC24hSettings settings)
        {
            this.mainActivity = mainActivity;
            this.settings = settings;
        }

        String readFully(InputStream entityResponse) throws IOException {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int length = 0;
            while ((length = entityResponse.read(buffer)) != -1) {
                baos.write(buffer, 0, length);
            }
            return baos.toString();
        }

        protected Long doInBackground(String... str)
        {
            String xml = "";
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(getBaseContext(), R.string.getting_settings, Toast.LENGTH_SHORT).show();
                }
            });

            try {
                String host = wc24hsettings.ipaddress;
                if (!host.startsWith("http://"))
                {
                    host = "http://" + host;
                }
                host += "/get_settings";
                URL url = new URL(host);
                HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
                try
                {
                    InputStream in = new BufferedInputStream(urlConnection.getInputStream());
                    xml = readFully(in);
                }
                catch (Exception e)
                {
                    final Exception ex = e;
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(getBaseContext(), getString(R.string.getting_settings_error) + ":\n" + ex.getLocalizedMessage(), Toast.LENGTH_LONG).show();
                        }
                    });
                    Log.e("WC24h", "Get settings from Wordclock: ", e);
                }
                finally
                {
                    urlConnection.disconnect();
                }

                if (xml.isEmpty()) {
                    return 0L;
                }

                //Log.w("WC24h", xml);
                XmlPullParser parser = Xml.newPullParser();
                try {
                    parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
                    parser.setInput(new StringReader(xml));
                    settings.loadData(parser);
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(getBaseContext(), R.string.getting_settings_success, Toast.LENGTH_SHORT).show();
                        }
                    });
                } catch (XmlPullParserException e) {
                    e.printStackTrace();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mainActivity.profileChanged(true);
                }
            });
            return null;
        }

        protected void onProgressUpdate(Integer... progress)
        {
        }

        protected void onPostExecute(Long result)
        {
        }
    }


    private class CommandSender extends AsyncTask<String, Integer, Long>
	{
        DatagramSocket s;

        CommandSender() {
            try {
                s = new DatagramSocket();
                s.setSoTimeout(100);
            }
            catch (final SocketException e)
            {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(getBaseContext(), "Socket Exception!: " + e.getLocalizedMessage(), Toast.LENGTH_SHORT).show();
                    }
                });
                // e.printStackTrace();
            }

        }
        void run() {
            if( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
                executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            } else {
                execute();
            }
        }
		protected Long doInBackground(String... str)
		{

		    try
			{

				String 	ipaddress 	= wc24hsettings.ipaddress;
			    int 	server_port = 2424;
				
				final InetAddress RemoteServer = InetAddress.getByName(ipaddress);  
			    DatagramPacket p = new DatagramPacket(global_messageBytes, global_msg_length, RemoteServer, server_port);  
			    s.send(p);
			    
			    /****************
			    byte[] receiveData = new byte[2];
			    s.setSoTimeout(500);   // set the timeout in millisecounds.
	            DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
			    
			    try
			    {
			    	s.receive(receivePacket);
			    	Toast.makeText(getBaseContext(), "Erfolgreich!", Toast.LENGTH_SHORT).show();
			    }
			    catch (SocketTimeoutException e)
			    {
			    	Toast.makeText(getBaseContext(), "Keine Antwort!", Toast.LENGTH_SHORT).show();
	            }
			    ***************/

			    s.close();
			}
			catch (SocketException e)
			{
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(getBaseContext(), "Socket Exception!", Toast.LENGTH_SHORT).show();
                    }
                });
				// e.printStackTrace();
			}
		    catch (UnknownHostException e)
			{
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(getBaseContext(), "Unknown Host!", Toast.LENGTH_SHORT).show();
                    }
                });
				// e.printStackTrace();
			}
		    catch (IOException e)
		    {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(getBaseContext(), "IO exception!", Toast.LENGTH_SHORT).show();
                    }
                });
				// e.printStackTrace();
			}
			return null;
		}

	    protected void onProgressUpdate(Integer... progress)
	    {
	    }

	    protected void onPostExecute(Long result)
	    {
	    }
	}

	private void myExit ()
	{
		Toast.makeText(getBaseContext(), "Bye bye...", Toast.LENGTH_SHORT).show();
		onDestroy();
		finish ();
		System.exit(0);
	}

	public void setMenusEnabled (boolean enabled)
	{
		if (menusCreated)
		{
			MenuItemEditServer.setEnabled(enabled);
		}
	}

	/** create the menu items */
	public void populateMenu(Menu menu)
	{
        MenuItem	MenuItemSync;
        MenuItem	MenuItemAbout;
        MenuItem	MenuItemATetris;
		MenuItem	MenuItemExit;
		// menu.setQwertyMode(true);

		MenuItemEditServer	= menu.add(0, MenuEditServer,	0, R.string.Settings);
        MenuItemSync		= menu.add(0, MenuSync, 		0, R.string.Synchronize);
        MenuItemATetris		= menu.add(0, MenuTetris, 		0, R.string.Tetris);
        MenuItemAbout		= menu.add(0, MenuAbout,		0, R.string.About);
		MenuItemExit		= menu.add(0, MenuExit,			0, R.string.Exit);

		menusCreated = true;
		setMenusEnabled (true);
        MenuItemSync.setEnabled(true);
        MenuItemAbout.setEnabled(true);
		MenuItemExit.setEnabled(true);
	}

	/** hook into menu button for activity */
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		populateMenu(menu);
		return super.onCreateOptionsMenu(menu);
	}

	/** when menu button option selected */
	@Override
	public boolean onOptionsItemSelected(MenuItem item)
	{
		return applyMenuChoice(item) || super.onOptionsItemSelected(item);
	}

	/** respond to menu item selection */
	public boolean applyMenuChoice(MenuItem item)
	{
		switch (item.getItemId())
		{
            case MenuEditServer:
            {
                editServer ();
                break;
            }

            case MenuSync:
            {
                sync();
                break;
            }

            case MenuAbout:
		    {
		    	about ();
		    	break;
		    }

            case MenuTetris:
            {
                TetrisDialog dlg = new TetrisDialog(this);
                dlg.show();
                break;
            }

            case MenuExit:
		    {
				myExit ();
		    	break;
		    }

		    default:
			{
				return false;
			}
		}
		return true;
	}

    private void sync ()
    {
        byte[]  messageBytes = new byte[7];

        Date now = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(now);

        messageBytes[0] = 'T';
        messageBytes[1] = (byte) (cal.get (Calendar.YEAR) - 2000);      // year since 2000
        messageBytes[2] = (byte) cal.get (Calendar.MONTH);              // 1..12
        messageBytes[3] = (byte) cal.get (Calendar.DAY_OF_MONTH);       // 1..31
        messageBytes[4] = (byte) cal.get (Calendar.HOUR_OF_DAY);        // 0..23
        messageBytes[5] = (byte) cal.get (Calendar.MINUTE);             // 0..59
        messageBytes[6] = (byte) cal.get (Calendar.SECOND);             // 0..59

        SendMessage (messageBytes, 7);
        Toast.makeText(getBaseContext(), R.string.date_time_sent, Toast.LENGTH_SHORT).show();

        messageBytes = new byte[2];
        messageBytes[0] = 'D';
        messageBytes[1] = (byte) wc24hsettings.getNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR);
        SendMessage(messageBytes, 2);

        messageBytes = new byte[2];
        messageBytes[0] = 'F';
        messageBytes[1] = (byte) wc24hsettings.getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR);
        SendMessage(messageBytes, 2);
        Toast.makeText(getBaseContext(), R.string.color_animation_mode_sent, Toast.LENGTH_SHORT).show();

        sendBrightness();
        sendAutoBrightness();
        sendDisplayColors();
    }

	private void about ()
    {
    	final AlertDialog.Builder alert = new AlertDialog.Builder(this);
		alert.setTitle(R.string.AboutWC24hTitle);
		alert.setMessage(R.string.AboutWC24hInfo);
        // alert.setIcon(R.drawable.icon);
        alert.setPositiveButton(R.string.OK, new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int whichButton)
			{
			}
		});
		alert.show();
    }

	private void editServer ()
	{
//	    final int			PADDING = 20;
//	    int					widthleft;
//	    int					widthright;
//
//        r = getResources();
//        widthleft = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 80, r.getDisplayMetrics()) + 0.5);
//        widthright = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 160, r.getDisplayMetrics()) + 0.5);
//
//	    AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
//
//        ScrollView sv = new ScrollView(this);
//        LinearLayout ll = new LinearLayout(this);
//        ll.setOrientation(LinearLayout.VERTICAL);
//
//    	LinearLayout xla = new LinearLayout(this);
//    	xla.setOrientation(LinearLayout.HORIZONTAL);
//
//    	TextView tva = new TextView(this);
//        tva.setText(R.string.ip_address_);
//        tva.setTextColor(Color.WHITE);
//        tva.setWidth(widthleft);
//        xla.addView(tva);
//
//        tvae = new EditText(this);
//        tvae.setText(wc24hsettings.ipaddress);
//        tvae.setTextColor(Color.WHITE);
//        tvae.setBackgroundColor(Color.TRANSPARENT);
//        tvae.setWidth(widthright);
//        // tvae.setInputType(InputType.TYPE_CLASS_PHONE);
//		xla.addView(tvae);
//        ll.addView(xla);
//
//        LinearLayout xla2 = new LinearLayout(this);
//        xla2.setOrientation(LinearLayout.HORIZONTAL);
//
//		ll.addView (xla2);
//
//        sv.addView(ll);
//
//        sv.setBackgroundColor(Color.rgb(25, 25, 25));
//        int padding_px = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, PADDING, r.getDisplayMetrics()) + 0.5);
//        sv.setPadding(padding_px, 0, padding_px, 0);
//        alertbox.setView (sv);
//
//	    alertbox.setPositiveButton(R.string.OK, new DialogInterface.OnClickListener()
//	    {
//	        public void onClick(DialogInterface arg0, int arg1)
//	        {
//				String ipaddress = tvae.getText().toString().trim();
//
//				if (ipaddress.contentEquals(""))
//				{
//					Toast.makeText(getBaseContext(), R.string.ip_address_must_be_set, Toast.LENGTH_SHORT).show();
//					ipaddress = "...";
//				}
//				else
//				{
//                    wc24hsettings.ipaddress = ipaddress;
//                    SharedPreferences prefs = getSharedPreferences("com.example.wc24h", Context.MODE_PRIVATE);
//                    int selectedProfile = prefs.getInt("com.example.wc24h.selected_profile", 0);
//                    wc24hsettings.saveData(getSettingsDirFile(MainActivity.this), getProfileFileName(selectedProfile));
//				}
//	        }
//	    });
//
//	    alertbox.setNegativeButton(R.string.Cancel, new DialogInterface.OnClickListener()
//	    {
//	        public void onClick(DialogInterface arg0, int arg1)
//	        {
//	            ;
//	        }
//	    });
//	    alertbox.show ();
        SharedPreferences prefs = getSharedPreferences("com.example.wc24h", Context.MODE_PRIVATE);
        int selectedProfile = prefs.getInt("com.example.wc24h.selected_profile", 0);
        File dir = MainActivity.getSettingsDirFile(this);
        ProfileDialog profileDialog = new ProfileDialog(this, dir, selectedProfile);
        profileDialog.show();
	}

	void sendAutoBrightness()
    {
        byte[] messageBytes = new byte[2];
        Boolean auto = wc24hsettings.getNumVar(WC24hSettings.DISPLAY_AUTOMATIC_BRIGHTNESS_ACTIVE_NUM_VAR) != 0;

        messageBytes[0] = 'L';

        if (auto)
        {
            messageBytes[1] = 1;
        }
        else
        {
            messageBytes[1] = 0;
        }

        SendMessage (messageBytes, 2);

        if (! auto)
        {
            messageBytes[0] = 'B';
            messageBytes[1] = (byte) wc24hsettings.getNumVar(WC24hSettings.DISPLAY_BRIGHTNESS_NUM_VAR);
            SendMessage (messageBytes, 2);
        }
    }

	public void myCheckBoxClickHandler(View view)
	{
		switch (view.getId())
		{
			case R.id.AutomaticBrightness:
			{
				byte[] messageBytes = new byte[2];
                boolean auto = automaticBrightness.isChecked();

				wc24hsettings.setNumVar(WC24hSettings.DISPLAY_AUTOMATIC_BRIGHTNESS_ACTIVE_NUM_VAR, (auto) ? 1 : 0);
                sendAutoBrightness();
				Toast.makeText(getBaseContext(), "Auto-Helligkeit gesandt!", Toast.LENGTH_SHORT).show();
	        	break;
			}
        }
	}

	public static String getProfileFileName(int idx)
    {
        return "WC24hProfile" + idx + ".xml";
    }

	public static File getSettingsDirFile(MainActivity activity)
    {
        File path = getExternalFilesDir(activity);
        File wc24hpath = new File(path, File.separator+"WC24h");
        if (!wc24hpath.exists()) {
            wc24hpath.mkdir();
        }

//        File wc24hpath;
//        File storageDir = new File("/mnt/extSdCard/WC24h");					// Samsung S3/S4/S5
//        if (storageDir.isDirectory ())
//        {
//            //Toast.makeText(activity, storageDir.getAbsolutePath() + activity.getString(R.string._found), Toast.LENGTH_SHORT).show();
//            Log.w("WC24h", storageDir.getAbsolutePath());
//            wc24hpath = storageDir;
//        }
//        else
//        {
//            wc24hpath = Environment.getExternalStoragePublicDirectory("WC24h");
//
//            if (wc24hpath.isDirectory ())
//            {
//                Log.w("WC24h", wc24hpath.getAbsolutePath());
//                //Toast.makeText(activity, wc24hpath.getAbsolutePath() + activity.getString(R.string._found), Toast.LENGTH_SHORT).show();
//            }
//            else if (! wc24hpath.mkdirs())
//            {
//                Toast.makeText(activity, wc24hpath + activity.getString(R.string._could_not_be_created), Toast.LENGTH_SHORT).show();
//                return null;
//            }
//        }
        return wc24hpath;
    }

    @SuppressLint("NewApi")
    public static File getExternalFilesDir(MainActivity activity) {
        String packageName = activity.getApplicationContext().getPackageName();
        File appFiles = null;

        try {
            //
            if (Build.VERSION.SDK_INT <= 7) {
                if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
                    File externalPath = Environment.getExternalStorageDirectory();
                    appFiles = new File(externalPath.getAbsolutePath() + "/Android/data/" + packageName + "/files");
                } else {
                    File internalPath = Environment.getDataDirectory();
                    appFiles = new File(internalPath.getAbsoluteFile() + "/data/" + packageName + "/files");
                }

                if (!appFiles.exists()) {
                    appFiles.mkdirs();
                }
            } else {
                appFiles = activity.getApplicationContext().getExternalFilesDir(null);

                if (appFiles == null || !appFiles.exists()) {
                    appFiles = activity.getApplicationContext().getFilesDir();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();

        }

        return appFiles;
    }
}

